import nx_utils as util

# Example graphs from class notes
sampleGraph1 = {1:[2,3,4], 2:[1,3,5], 3:[1,2], 4:[1,7,9], 5:[2,9], 6:[9], 7:[4], 8:[9], 9:[5,8,6,4]}
sampleGraph2 = {1:[2,3,4], 2:[1,3,5], 3:[1,2], 4:[1,7,9], 5:[2,9], 6:[9], 7:[4], 8:[9], 9:[5,8,6,4], 10:[11, 12], 11:[10], 12:[10]}

# util.plot_graph(sampleGraph1, "SampleGraph1")
# util.plot_graph(sampleGraph2, "SampleGraph2")

def dfs(graph,source,target):
    """Returns the target node, if it is connected to the source node in the dictionary-type graph; 
    otherwise returns None.
    """

    # A local set to store which nodes have been visited by def_help().
    # This needs to be initialized before each call to def_help().
    visited = set()

    def dfs_help(graph,source,target):
        """ Helper function that returns the target node, if it is connected to the 
        source node in the graph; otherwise returns None.  Utilizes the 
        visited set in its closure. 
        """
        
        if source in visited:
            # Already been here.  Don't try again.
            print "Been here already: ", source
            return None

        else:
            print "We're at node",source

            # Remember that we've come here.
            visited.add(source)

            if source == target:
                # Found it!  We're done.
                print "Found target = ", target
                return target

            else:
                # Keep looking from where we're at.
                print "Searching node ", source, " neighbors: ", graph[source]
                for node in graph[source]:
                    result = dfs_help(graph,node,target)  #recurse.  Note that result is further processed below.

                    if result != None:
                        # Found it!  We're done.
                        print "Found result = ", result    
                        return result
                    else:
                        print "No path found from ",node," to ", target
                    
                # Not reachable from here via any edge.
                print "Can't find it in: ", graph[source]
                return None
                
    return dfs_help(graph, source, target)


def dfs2(graph,source,target):
    """Returns the target node, if it is connected to the source node in the dictionary-type graph; 
    otherwise returns None.
    """
    # A local set to store which nodes have been visited by def_help().
    # This needs to be initialized before each call to def_help().
    visited = set()

    def dfs_help(graph,sources,target):
        """ Helper function that returns the target node, if it is connected to one 
        of the given source nodes in the graph; otherwise returns None.
        Utilizes the visited set in its closure.
        Tail-recursive algorithm with no loops and fewer conditional checks.
        """
        
        print "Searching nodes: ", sources
        if sources == []:
            # base case.  Done.  Looked everywhere, so no path to target
            print "Can't find it!"
            return None
            
        print "We're at node",sources[0]            
        if sources[0] in visited:
            # Already been here.  Try nextNodes source
            print "Been here already: ", sources[0]
            return dfs_help(graph, sources[1:], target)  # recurse.  Note that result is not further processed (tail recursive call)

        else:
            # Remember that we've come here.
            visited.add(sources[0])

            if sources[0] == target:
                # Found it!  We're done.
                print "Found target = ", target
                return target

            else:
                # Keep looking from where we're at.
                nextNodes = graph[sources[0]]+sources[1:]   # prepend the neigbbors onto the rest of the source nodes
                return dfs_help(graph, nextNodes, target)   # recurse.  Note that result is not further processed (tail recursive call)
                
    return dfs_help(graph, [source], target)


def dfsNX(nxGraph,source,target):
    """Returns the target node, if it is connected to the source node in the networkx-type graph; 
    otherwise returns None.   Utlizes the node attribute "visited" to mark the nodes.  The 
    visited attribute is cleared in the whole graph before the algorithm is run.
    """

    
    # clear the visited node attribute
    for n in nxGraph.nodes():
        nxGraph.node[n]["visited"] = False
        
    def dfs_help(nxGraph,sources,target):
        """ Helper function that returns the target node, if it is connected to 
        one of the given source nodes in the nxGraph; otherwise returns None.
        """
        
        print "Searching nodes: ", sources
        if sources == []:
            # base case.  Done.  Looked everywhere, so no path to target
            print "Can't find it!"
            return None
            
        print "We're at node",sources[0]            
        if nxGraph.node[sources[0]]["visited"]:    # Check if the node is marked.
            # Already been here.  Try nextNodes source
            print "Been here already: ", sources[0]
            return dfs_help(nxGraph, sources[1:], target)  # recurse. Note that result is not further processed (tail recursive call)

        else:
            # Remember that we've come here.
            nxGraph.node[sources[0]]["visited"] = True   # mark the node

            if sources[0] == target:
                # Found it!  We're done.
                print "Found target = ", target
                return target

            else:
                # Keep looking from where we're at.
                nextNodes = nxGraph.neighbors(sources[0])+sources[1:]   # prepend the neigbbors onto the rest of the source nodes
                return dfs_help(nxGraph, nextNodes, target)   # recurse. Note that result is not further processed (tail recursive call)
                
    return dfs_help(nxGraph, [source], target)
    